package dasher.android;

import java.util.List;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.PowerManager;
import dasher.CDasherInput;
import dasher.CDasherInterfaceBase;
import dasher.CDasherView;
import dasher.CParameterNotificationEvent;
import dasher.CSettingsStore;
import dasher.Ebp_parameters;

public class TiltInput extends CDasherInput implements SensorEventListener {
	private final SensorManager sm;
	
	private float fx,fy;
	private boolean bActive;
	private final Sensor s;
	
	public static TiltInput MAKE(Context androidCtx, CDasherInterfaceBase iface, CSettingsStore sets) {
		SensorManager sm=(SensorManager)androidCtx.getSystemService(Context.SENSOR_SERVICE);
		
		List<Sensor> ss = sm.getSensorList(Sensor.TYPE_ACCELEROMETER);
		if (ss.isEmpty()) return null;
		return new TiltInput(iface, sets, sm, ss.get(0));
	}
	
	private TiltInput(CDasherInterfaceBase iface, CSettingsStore sets, SensorManager sm, Sensor s) {
		super(iface, sets, 1, "Tilt Input");
		this.sm=sm;
		this.s=s;
	}
	
	@Override public void Activate() {
		if (bActive) return;
		bActive=true;
		sm.registerListener(this, s, SensorManager.SENSOR_DELAY_UI);
	}
	
	@Override public void Deactivate() {
		if (!bActive) return;
		bActive=false;
		sm.unregisterListener(this,s);
	}
	
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	public void onSensorChanged(SensorEvent event) {
		float[] vals=event.values;
		/*StringBuilder sb=new StringBuilder();
		for (int i=0; i<vals.length; i++)
			sb.append(i==0 ? "{" : ",").append(vals[i]);
		sb.append("}");
		android.util.Log.d("DasherIME","Got rotation "+sb);*/
		fx = Math.max(0.0f, Math.min(1.0f, (vals[0]-1.0f)/-2.0f));
		fy = Math.max(0.0f, Math.min(1.0f, (vals[1]-1.0f)/8.0f));
		
	}

	@Override
	public boolean GetScreenCoords(CDasherView pView,long[] Coordinates) {
		if (!bActive) return false;
		Coordinates[0] = (int)(fx * pView.Screen().GetWidth());
		Coordinates[1] = (int)(fy * pView.Screen().GetHeight());
		//android.util.Log.d("DasherIME","Tilt: got "+Coordinates[0]+", "+Coordinates[1]);
		return true;
	}

}
